# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action navigation_filters
#
# Edite les differentes regles de filtrage
#
###########################################################################

""" Action de gestion des filtres web sur un amon """
import commands, os
from os.path import isfile, join

from twisted.python import log

from ead2.backend.lib.action import Action, Dict
from ead2.backend.actions import tools
import ead2.backend.actions.amon.amon_tools as t
from ead2.backend.actions.amon import forms
from ead2.backend.actions.amon import whitesitelist_forms
from ead2.backend.actions.amon import menus
from ead2.backend.config.config import SCRIPT_DIR, CONFIG_DIR
from ead2.backend.config.filenames import navigation_filters
from ead2.lib.libead import uni
from ead2.lib.error import MissingKey, MissingValue

used_templates = ['main', 'form', 'formselect', 'accordion']
zone_libelle = t.get_zone_libelle(1)

signalement_msg = navigation_filters['signalement_msg']

class FiltresWebadmin(Action):
    """ gestion des filtres optionnels de dansguardian pour la zone admin"""
    user_description = Dict(default={},
                            doc="description de l'éxécutant",
                            keys=['ip', 'name', 'role'])
    name = 'opt_filters_admin'
    libelle = "Sites"
    description = "Activation des listes de sites optionnelles sur '%s'" % (
                                                        zone_libelle.lower(),)
    category = zone_libelle.capitalize()
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={},
                       doc="retour formulaire de la requete en cours coté frontend",
                       keys = ['opt_filters_form'])
    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        if self.form_result.has_key('opt_filters_form'):
            try:
                datas = forms.valid_opt_filters_form(self.form_result['opt_filters_form'],
                                                     'admin')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_opt_filters', templates=used_templates)
        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'filtres', 'admin')
        result.update(forms.get_opt_filters_form('admin'))
        result['validate'] = self.valid_btn()
        return self.send_all(result, template='amon_opt_filters', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['opt_filters_form'],
                                    'msg_container')
        title = 'Modifier la configuration des filtres'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class Filtrageadmin(Action):
    """ réglage du mode de filtrage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'filtrage_admin'
    libelle = "Mode de filtrage pour la navigation sur '%s'" % (
                                            zone_libelle.lower(),)
    description = "Mode de filtrage pour la navigation sur '%s'" % (
                                            zone_libelle.lower(),)
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['filtrage_form'])
    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        if self.form_result.has_key('filtrage_form'):
            try:
                datas = forms.valid_filtrage_form(self.form_result['filtrage_form'], 'admin')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_filtrage', templates=used_templates)
        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'filtrage', 'admin')
        result.update(forms.get_filtrage_form('admin'))
        result['validate'] = self.valid_btn()
        return self.send_all(result, template='amon_filtrage', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['filtrage_form'],
                                    'msg_container')
        title = 'Modifier les règles de filtrage'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)


class SitesInterditsadmin(Action):
    """ ajout de domaines interdits à la liste """
    user_description = Dict(default={}, doc="description de l'exécutant",
                            keys=['ip', 'name', 'role'])
    name = 'sites_interdits_admin'
    libelle = "Gestion des domaines interdits sur '%s'" % (
                                   zone_libelle.lower(),)
    description = libelle
    category = None
    request = Dict(default={},
                    doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={},
                       doc="retour formulaire de la requete en cours coté frontend",
                       keys = ['sitelist_form', 'site_names', 'site_table'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('sitelist_form'):
            try:
                datas = forms.valid_sitelist_form(self.form_result['sitelist_form'],
                                                   'admin',
                                                   'blacklist')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        elif self.form_result.has_key('site_table') and self.form_result.has_key('site_names'):
            try:
                datas = forms.valid_mod_list(self.form_result['site_names'],
                                                  self.form_result['site_table'],
                                                  'admin',
                                                  'blacklist')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'blacklist', 'admin')
        result.update(forms.get_blacklist_form('admin'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_listed('admin', 'blacklist'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_sitelist', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['sitelist_form'],
                                    'msg_container')
        title = 'Ajouter des domaines à la liste des domaines interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
            (de la section de modification de blacklist locale)
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['site_names',
                                     'site_table'
                                    ],
                                    'msg_container')
        title = 'Modifier la liste de domaines interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class SitesAutorisesadmin(Action):
    """ ajout de domaines autorisés à la liste """
    user_description = Dict(default={}, doc="description de l'exécutant",
                            keys=['ip', 'name', 'role'])
    name = 'sites_autorises_admin'
    libelle = "Gestion des domaines autorisés sur '{0}'".format(zone_libelle.lower())
    description = libelle
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={},
                       doc="retour formulaire de la requete en cours coté frontend",
                       keys = ['sitelist_form', 'site_table', 'site_names'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('sitelist_form'):
            try:
                datas = forms.valid_sitelist_form(self.form_result['sitelist_form'],
                                                  'admin',
                                                  'whitelist')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        elif self.form_result.has_key('site_table') and self.form_result.has_key('site_names'):
            try:
                datas = forms.valid_mod_list(self.form_result['site_names'],
                                                  self.form_result['site_table'],
                                                  'admin',
                                                  'whitelist')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'whitelist', 'admin')
        result.update(forms.get_whitelist_form('admin'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_listed('admin', 'whitelist'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_sitelist', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['sitelist_form'],
                                    'msg_container')
        title = 'Ajouter des domaines à la liste de domaines autorisés'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
            (de la section de modification de blacklist locale)
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['site_names',
                                     'site_table'],
                                    'msg_container')
        title = 'Modifier la liste de domaines autorisés'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class Extensionsadmin(Action):
    """
        gestion des extensions interdites sur le filtre
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'extensions_admin'
    libelle = 'Extensions'
    description = "Ajouter des extensions à interdire sur '{0}'".format(zone_libelle.lower())
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['extension_form', 'extlist_names', 'ext_table'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('extension_form'):
            try:
                datas = forms.valid_extension_form(self.form_result['extension_form'], 'admin')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_extension', templates=used_templates)

        elif self.form_result.has_key('extlist_names') and self.form_result.has_key('ext_table'):
            try:
                datas = forms.valid_mod_list(self.form_result['extlist_names'],
                                                  self.form_result['ext_table'],
                                                  'admin',
                                                  'extension')
                datas['toexec'] = tools.make_js_link(self.server_nb,
                                                     self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_extension', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'extension', 'admin')
        result.update(forms.get_extension_form('admin'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_forbidden_extension('admin'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_extension', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['extension_form'],
                                    'msg_container')
        title = 'Ajouter des sites à la liste des extensions à interdire'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['extlist_names',
                                     'ext_table'],
                                    'msg_container')
        title = 'Modifier la liste des domaines interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class Mimeadmin(Action):
    """
        gestion des mime interdites sur le filtre web
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'mime_admin'
    libelle = 'Types Mime'
    description = "Ajouter des types mimes à interdire sur '{0}'".format(zone_libelle.lower())
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['mime_form', 'mime_names', 'mime_table'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('mime_form'):
            try:
                datas = forms.valid_mime_form(self.form_result['mime_form'], 'admin')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_mime', templates=used_templates)

        elif self.form_result.has_key('mime_names') and self.form_result.has_key('mime_table'):
            try:
                datas = forms.valid_mod_list(self.form_result['mime_names'],
                                                  self.form_result['mime_table'],
                                                  'admin',
                                                  'mime')
                datas['toexec'] = tools.make_js_link(self.server_nb,
                                                     self.name,
                                                     confirm=False)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_mime', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'mime', 'admin')
        result.update(forms.get_mime_form('admin'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_forbidden_mimes('admin'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_mime', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['mime_form'],
                                    'msg_container')
        title = 'Ajouter des types mimes à la liste des types mimes à interdire'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['mime_names',
                                        'mime_table'],
                                    'msg_container')
        title = 'Modifier la liste de types mimes à interdire.'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)


class NavigationWhiteSiteListadmin(Action):
    """ interface de gestion des utilisateurs(ips) en liste blanche
        sur l'instance 0 de dansguardian
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_whitesitelist_admin'
    libelle = 'Sites liste blanche'
    description = "Ajouter des sites pour le mode liste blanche sur '%s'" % (zone_libelle.lower(),)
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['list_suppr', 'list_add'])

    def execute(self):
        """ renvoit les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('list_add'):
            try:
                datas = whitesitelist_forms.valid_add_greysite(
                                            self.form_result['list_add'],
                                            zone='admin')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                                  confirm=True)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_whitesitelist',
                                         templates=used_templates)

        elif self.form_result.has_key('list_suppr'):
            try:
                datas = whitesitelist_forms.valid_mod_greysitelist(self.form_result['list_suppr'],
                                                       zone='admin')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, msg:
                log.err()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_whitesitelist',
                                         templates=used_templates)

        result = {'titre': self.description}
        result.update(whitesitelist_forms.get_add_site_form(zone='admin'))
        result.update(whitesitelist_forms.get_greysitelist(zone='admin'))
        result['sstitre'] =  ''
        result['menus'] = menus.get_filters_menus(self.server_nb,
                                          'whitesitelist',
                                          'admin')
        result['validate'] = whitesitelist_forms.get_validate(self.server_nb, self.name)
        result['validate_mod'] = whitesitelist_forms.get_validate_mod(self.server_nb, self.name)
        return self.send_all(result, template='amon_whitesitelist',
                                     templates=used_templates)

